package game.command;

public class IntParser extends DefaultingParser {
	private final Integer min;
	private final Integer max;
	private final boolean hex;
	
	public IntParser(String name, boolean hex, Integer def, Integer min, Integer max, Object ... completions) {
		super(name, def, completions);
		this.min = min;
		this.max = max;
		this.hex = hex;
	}
	
	public Integer parse(ScriptEnvironment env, String input) {
		int value;
		try {
			value = Integer.parseInt(input, this.hex ? 16 : 10);
		}
		catch(NumberFormatException e) {
			throw new ScriptException("Ungültige " + (this.hex ? "Hexadezimalzahl" : "Ganzzahl") + " '%s'", input);
		}
		if(this.min != null && value < this.min)
			if(this.max != null)
				throw new ScriptException("Die Zahl muss im Bereich %d .. %d liegen, habe %d", this.min, this.max, value);
			else
				throw new ScriptException("Die Zahl muss mindestens %d betragen, habe %d", this.min, value);
		if(this.max != null && value > this.max)
			if(this.min != null)
				throw new ScriptException("Die Zahl muss im Bereich %d .. %d liegen, habe %d", this.min, this.max, value);
			else
				throw new ScriptException("Die Zahl darf höchstens %d betragen, habe %d", this.max, value);
		return value;
	}

	public Class<?> getTypeClass() {
		return this.hasDefault() ? int.class : Integer.class;
	}
}
