package game.command;

import java.util.List;

import com.google.common.collect.Lists;

import game.dimension.Dimension;
import game.world.WorldServer;

public class WorldParser extends DimensionParser {
	private final boolean loadedOnly;
	
	public WorldParser(String name, boolean loadedOnly, boolean useSender) {
		super(name, useSender);
		this.loadedOnly = loadedOnly;
	}

	public WorldServer parse(ScriptEnvironment env, String input) {
		Dimension dim = (Dimension)super.parse(env, input);
		WorldServer world = this.loadedOnly ? env.getServer().getWorldNoLoad(dim.getDimensionId()) : env.getServer().getWorld(dim.getDimensionId());
		if(world == null)
			throw new ScriptException("Dimension '%s' ist nicht geladen", dim.getFormattedName(false));
		return world;
	}
	
	public WorldServer getDefault(ScriptEnvironment env) {
		Dimension dim = (Dimension)super.getDefault(env);
		return this.loadedOnly ? env.getServer().getWorldNoLoad(dim.getDimensionId()) : env.getServer().getWorld(dim.getDimensionId());
//		if(world == null)
//			throw new ScriptException("Dimension '%s' ist nicht geladen", dim.getFormattedName(false));
//		return world;
	}
	
	public String[] getCompletions(ScriptEnvironment env) {
		if(this.loadedOnly) {
			List<String> loaded = Lists.newArrayList();
			for(WorldServer world : env.getServer().getWorlds()) {
				loaded.add(world.dimension.getDimensionName());
			}
			return loaded.toArray(new String[loaded.size()]);
		}
		return super.getCompletions(env);
	}

	public Class<?> getTypeClass() {
		return WorldServer.class;
	}
}
