package game.command.commands;

import java.util.Set;

import com.google.common.collect.Sets;

import game.command.ScriptEnvironment;
import game.command.ScriptException;
import game.command.ScriptExecutable;
import game.command.ScriptExecutor;
import game.entity.Entity;
import game.entity.types.EntityLiving;
import game.init.EntityRegistry;
import game.nbt.NBTTagCompound;
import game.world.Vec3;
import game.world.WorldServer;

public class CommandSpawn extends ScriptExecutable {
	public CommandSpawn() {
		super("spawn");
		
		Set<String> names = Sets.newTreeSet();
		for(Class<? extends Entity> clazz : EntityRegistry.getAllClasses()) {
			names.add(EntityRegistry.getEntityString(clazz));
		}
		this.addEnum("type", String.class, names);
		this.setParamsOptional();
		this.addVector("position", true);
		this.addWorld("dim", true);
		this.addTag("tag");
		
		this.addFlag("noinit", 'n');
		this.addInt("count", 'c', 1, 1024, 1);
		this.addTag("postTag", 'p');
	}
	
	public Object exec(ScriptEnvironment env, ScriptExecutor exec, String type, Vec3 pos, WorldServer world, NBTTagCompound tag, boolean noinit, int count, NBTTagCompound postTag) {
		for(int z = 0; z < count; z++) {
			Entity entity = EntityRegistry.createEntityByName(type, world);
			if(entity == null)
				throw new ScriptException("Objekt konnte nicht erzeugt werden");
			entity.setLocationAndAngles(pos.xCoord, pos.yCoord, pos.zCoord, world.rand.floatv() * 360.0f, 0.0f);
			if(tag != null) {
				NBTTagCompound ent = new NBTTagCompound();
				entity.writeToNBT(ent);
				ent.merge(tag);
				entity.readFromNBT(ent);
			}
			if(!noinit && (entity instanceof EntityLiving))
				((EntityLiving)entity).onInitialSpawn(null);
			world.spawnEntityInWorld(entity);
			if(postTag != null) {
				NBTTagCompound ent = new NBTTagCompound();
				entity.writeToNBT(ent);
				ent.merge(postTag);
				entity.readFromNBT(ent);
			}
		}
		exec.logConsole("%s%s bei %d, %d, %d in %s erschaffen", count == 1 ? "" : (count + "x "), EntityRegistry.getEntityName(type), (int)pos.xCoord, (int)pos.yCoord, (int)pos.zCoord, world.dimension.getFormattedName(false));
		return null;
	}
}
