package game.gui;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.Arrays;
import javax.imageio.ImageIO;

import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL13;

import game.Game;
import game.entity.npc.Alignment;
import game.entity.npc.CharacterInfo;
import game.entity.npc.EntityNPC;
import game.entity.npc.SpeciesInfo;
import game.entity.types.EntityLiving;
import game.gui.element.ActButton;
import game.gui.element.ActButton.Mode;
import game.gui.element.Element;
import game.gui.element.GuiList;
import game.gui.element.Label;
import game.gui.element.ListEntry;
import game.gui.element.Slider;
import game.gui.element.Textbox;
import game.gui.element.Textbox.Action;
import game.init.EntityRegistry;
import game.init.SpeciesRegistry;
import game.init.SpeciesRegistry.ModelType;
import game.log.Log;
import game.network.NetHandlerPlayServer;
import game.packet.CPacketAction;
import game.packet.CPacketMessage;
import game.packet.CPacketSkin;
import game.renderer.Drawing;
import game.renderer.GlState;
import game.renderer.ItemRenderer;
import game.renderer.entity.Render;
import game.renderer.entity.RenderManager;
import game.renderer.texture.EntityTexManager;
import game.renderer.texture.TextureUtil;
import game.util.FileUtils;
import game.util.SkinConverter;
import game.window.Button;

public class GuiSkin extends GuiList<GuiSkin.SkinEntry>
{
	protected class SkinEntry implements ListEntry
	{
	    private final File skinFile;
	    private final String id;
	    private final ModelType model;
	    private final CharacterInfo charinfo;
	    private final BufferedImage skinImage;
//	    private final String icon;
//	    private DynamicTexture dyntex;
	    private final int dynId;

	    protected SkinEntry(String id, File file, CharacterInfo charinfo, BufferedImage image, ModelType model)
	    {
//	        this.owner = parent;
	        this.skinFile = file;
	    	this.charinfo = charinfo;
	    	this.id = id;
	        this.model = model;
//	        this.gm = Game.getGame();
	        if(this.skinFile != null) // {
	            this.skinImage = image;
//	            this.skin = skin.getName().substring(0, skin.getName().length() - 4);
//	        }
	        else // {
	            this.skinImage = null;
//	        	this.skin = null;
//	        }
	        if(this.skinImage != null) {
	            int w = this.skinImage.getWidth();
	            int h = this.skinImage.getHeight();
	            int[] data = new int[w * h];
	            this.skinImage.getRGB(0, 0, w, h, data, 0, w);
	            this.dynId = 0xffff0000 + GuiSkin.this.elements.size();
	        	EntityTexManager.setTexture(this.dynId, EntityTexManager.imageToComp(data, model), model);
	        }
	        else {
	        	this.dynId = -1;
	        }
	    }

	    public void draw(int x, int y, int mouseX, int mouseY, boolean hovered)
	    {
//	        FontRenderer.drawString(this.skin == null ? "---" : this.skin, x + 32 + 3, y + 1, 16777215);
	        String str = // List<String> list = FontRenderer.listFormattedStringToWidth(
	        		(this.skinFile != null ? this.skinFile.getName() 
	        		: (this.charinfo == null ? "Standard" : (
	        				(this.charinfo.species.prefix && this.charinfo.spclass != null && this.charinfo.spclass.type != null ? 
	        				this.charinfo.spclass.type.toString() : 
	        				EntityRegistry.getEntityName(this.charinfo.species.id))
	        				+ (this.charinfo.name.isEmpty() ? "" : (" " + this.charinfo.name)) + " (" + this.charinfo.skin + ")")));
//	        		+ TextColor.LIGHT_GRAY + " (" + TextColor.GREEN +
//	        		this.model.display + TextColor.LIGHT_GRAY +")"
//	        		, listWidth - 16 - 2);
//
//	        for (int i = 0; i < Math.min(list.size(), 2); ++i)
//	        {
            Drawing.drawText(str, x + 64 + 3, y, 0xff000000 | (this.charinfo == null ? 
            		(this.skinFile != null ? 0xffffff : 0xc0c0c0) : this.charinfo.color1 | this.charinfo.color2));
//	        }

	        GlState.color(1.0F, 1.0F, 1.0F, 1.0F);

//	        if(this.model != ModelType.HUMANOID) {
//	        	this.drawTextureAt(x, y, EntitySkinManager.HEAD_DEF);
//	        }
	        
	        if (/* this.gm.touchscreen || */ hovered)
	        {
//	        	GuiSkin.this.gm.getTextureManager().bindTexture(SKINS_BUTTONS);
	            Drawing.drawRect(x, y, 64, 64, -1601138544);
//	            GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
//	            int k1 = mouseX - x;
//	            int l1 = mouseY - y;
//
//	            if (k1 < 16 && k1 > 0)
//	            {
//	                Gui.drawScaledCustomSizeModalRect(x, y, 0.0F, 32.0F, 32, 32, 16, 16, 256.0F, 256.0F);
//	            }
	        }
	        
	        if (this.dynId != -1)
	        {
	            this.drawTextureAt(x, y, EntityTexManager.getSkin(this.dynId, this.model));
	        }
	        else if(this.charinfo == null)
	        {
	            this.drawTextureAt(x, y, EntityTexManager.getDefault(this.model));
	        }
	        else
	        {
	            this.drawTextureAt(x, y, EntityTexManager.getNpcSkin(this.charinfo.skin, this.model));
	        }
	    }

	    protected void drawTextureAt(int x, int y, String tex)
	    {
//	    	GuiSkin.this.gm.getTextureManager().bindTexture(tex);
	        GlState.enableBlend();
	        GlState.enableDepth();
	        Render.drawNames = false;
	        EntityTexManager.altTexture = tex;
	        EntityTexManager.altLayer = this.dynId;
	        EntityTexManager.altNpcLayer = this.dynId == -1 && this.charinfo != null ? this.charinfo.skin : null;
	        drawEntity(x + 32, y + 60, 28.0f 
	        		/ GuiSkin.this.gm.thePlayer.getHeight(), -45.0f, -20.0f, GuiSkin.this.gm.thePlayer);
	        Render.drawNames = true;
	        EntityTexManager.altTexture = null;
	        EntityTexManager.altLayer = -1;
	        EntityTexManager.altNpcLayer = null;
//	        GuiSkin.this.gm.getRenderManager().getRenderObject(this.model).renderHead(x, y, this.model.texWidth, this.model.texHeight, 2);
	        GlState.disableBlend();
	        GlState.disableDepth();
	    }
	    
	    public void deleteTexture() {
	    	if(this.dynId != -1) {
	    		EntityTexManager.setTexture(this.dynId, null, null);
	    	}
	    }

	    public void select(boolean dclick, int mx, int my)
	    {
//	    	if(GuiSkin.this.adjust.dragging)
//	    		return;
	    	if (dclick)
	        {
	    		GuiSkin.this.gm.displayGuiScreen(GuiMenu.INSTANCE);
	    		return;
	        }
	        else {
	            BufferedImage img = this.skinImage;
//	            boolean slim = false;
	            if(this.charinfo != null) {
	            	try {
	            		img = TextureUtil.readImage(FileUtils.getResource(
	            				EntityNPC.getSkinTexture(this.charinfo.skin.startsWith("~") ? this.charinfo.skin.substring(1) : 
	            				this.charinfo.skin)));
	            	}
	            	catch(IOException e) {
	            		if(e instanceof FileNotFoundException)
							Log.JNI.warn("Textur für Skin ist nicht vorhanden: " + 
							EntityNPC.getSkinTexture(this.charinfo.skin.startsWith("~") ? this.charinfo.skin.substring(1) : 
							this.charinfo.skin));
						else
							Log.JNI.error(e, "Konnte Textur nicht laden");
	            		return;
	            	}
//	            	slim = this.charinfo.skin.startsWith("~");
	            }
//	            else if(this.skinFile != null) {
//	            	slim = this.skinFile.getName().startsWith("slim_");
//	            }
//	            GuiSkin.this.selectEntry(slotIndex, slim);
	            GuiSkin.this.convertButton1.enabled = this.canConvert();
	            GuiSkin.this.convertButton2.enabled = this.canConvert();
	            GuiSkin.this.templateButton.enabled = this.canCopy();
	            GuiSkin.this.selectSkin(img, this.model);
	    		if(mx < 16 && mx > 0)
	    			GuiSkin.this.gm.displayGuiScreen(GuiMenu.INSTANCE);
	            return;
	        }
	    }

	    public File getFile()
	    {
	        return this.skinFile;
	    }

	    public boolean canConvert()
	    {
	        return this.skinFile != null && this.model == ModelType.HUMANOID;
	    }

	    public String getLocation()
	    {
	        return this.charinfo == null ? null : (this.charinfo.skin.startsWith("~") ? this.charinfo.skin.substring(1) : this.charinfo.skin);
	    }

	    public boolean canCopy()
	    {
	        return this.charinfo != null;
	    }
	}
	
	private class DragAdjust extends Element {
		private int mouseX;
		private int mouseY;
	    private float yawOffset;
	    private float pitchOffset;
//		public boolean dragging;

		public DragAdjust(int x, int y, int w, int h) {
			super(x, y, w, h, null);
		}

		protected void drawBackground() {
//			if(this.dragging) {
//				
//			}
			Drawing.drawRect(this.pos_x, this.pos_y, this.size_x, this.size_y, 0x20ffffff);
		}
		
		protected void drawForeground(int x1, int y1, int x2, int y2) {
		}
		
		public void drag(int x, int y) {
			GuiSkin.this.yaw = this.yawOffset + (this.mouseX - x) * 2.0f;
			GuiSkin.this.pitch = this.pitchOffset + (this.mouseY - y) * 2.0f;
		}

		public void mouse(Button btn, int x, int y, boolean ctrl, boolean shift) {
			this.mouseX = x;
			this.mouseY = y;
			this.yawOffset = GuiSkin.this.yaw;
			this.pitchOffset = GuiSkin.this.pitch;
//			this.dragging = true;
		}

//		public void mouserel() {
//			this.dragging = false;
//		}

		public boolean canHover() {
			return false;
		}
	}
	
	public static final GuiSkin INSTANCE = new GuiSkin();

    private ActButton convertButton1;
    private ActButton convertButton2;
    private ActButton templateButton;
    private DragAdjust adjust;
    private float yaw = -15.0f;
    private float pitch = -15.0f;
    
    private GuiSkin() {
    }
    
    public void init(int width, int height)
    {
    	super.init(width, height);
        this.setDimensions(400, height, 52, height - 52);
        this.load(null, this.gm.thePlayer == null ? ModelType.HUMANOID : this.gm.thePlayer.getModel());
        this.convertButton1 = this.add(new ActButton(10, height - 48, 200, 20, new ActButton.Callback() {
			public void use(ActButton elem, Mode action) {
				SkinEntry skin = GuiSkin.this.getSelected();
				if(skin != null && skin.getFile() != null && SkinConverter.convertSkin(skin.getFile(), new File("skins"), false))
					GuiSkin.this.gm.displayGuiScreen(GuiSkin.this);
			}
		}, "Konvertieren: Standard"));
        this.convertButton2 = this.add(new ActButton(10, height - 24, 200, 20, new ActButton.Callback() {
			public void use(ActButton elem, Mode action) {
				SkinEntry skin = GuiSkin.this.getSelected();
				if(skin != null && skin.getFile() != null && SkinConverter.convertSkin(skin.getFile(), new File("skins"), true))
					GuiSkin.this.gm.displayGuiScreen(GuiSkin.this);
			}
		}, "Konvertieren: Schlank"));
        this.add(new ActButton(240, height - 48, 150, 20, new ActButton.Callback() {
			public void use(ActButton elem, Mode action) {
				GuiSkin.this.gm.displayGuiScreen(GuiSkin.this);
			}
		}, "Neu laden"));
        this.templateButton = this.add(new ActButton(240, height - 24, 150, 20, new ActButton.Callback() {
			public void use(ActButton elem, Mode action) {
				SkinEntry skin = GuiSkin.this.getSelected();
				if(skin != null && skin.getLocation() != null) {
            		String loc = skin.getLocation();
            		File file = new File(new File("skins"), loc + ".png");
            		int z = 1;
            		while(file.exists()) {
            			file = new File(new File("skins"), loc + "_" + (++z) + ".png");
            		}
            		InputStream in = null;
            		try {
            			in = FileUtils.getResource(EntityNPC.getSkinTexture(loc));
            			Files.copy(in, file.toPath());
            		}
            		catch(Exception e) {
            			if(e instanceof FileNotFoundException)
							Log.JNI.warn("Textur ist nicht zum Kopieren vorhanden: " + EntityNPC.getSkinTexture(loc));
						else
							Log.JNI.error(e, "Konnte Textur nicht kopieren");
            		}
            		finally {
						if(in != null) {
							try {
								in.close();
							}
							catch(Throwable e) {
							}
						}
					}
            		GuiSkin.this.gm.displayGuiScreen(GuiSkin.this);
            	}
			}
		}, "Vorlage kopieren"));
        this.adjust = this.add(new DragAdjust(400 + 10, 64, width - 400 - 400 - 20, height - 128));
        
//        final ActButton[] speciesBtns = new ActButton[SpeciesRegistry.SPECIMEN.size()];
        for(int z = 0; z < SpeciesRegistry.SPECIMEN.size(); z++) {
        	final SpeciesInfo species = SpeciesRegistry.SPECIMEN.get(z);
//        	speciesBtns[z] = 
        	this.add(new ActButton(width - 400 + (z % 2) * 196, 4 + 23 * (z / 2), 194, 20, new ActButton.Callback() {
    			public void use(ActButton elem, Mode action) {
                	if(GuiSkin.this.gm.thePlayer != null) {
                		GuiSkin.this.gm.displayGuiScreen(GuiMenu.INSTANCE);
                		GuiSkin.this.gm.thePlayer.sendQueue.addToSendQueue(new CPacketAction(CPacketAction.Action.SET_SPECIES, EntityRegistry.getEntityID(species.clazz)));
//                        for(ActButton btn : speciesBtns) {
//                        	btn.enabled = btn != elem;
//                        }
                	}
    			}
    		}, EntityRegistry.getEntityName(EntityRegistry.getEntityString(species.clazz)))) // ;
//        	speciesBtns[z]
        			.enabled = this.gm.thePlayer == null || this.gm.thePlayer.getClass() != species.clazz;
        }
        
        final ActButton[] alignBtns = new ActButton[Alignment.values().length];
        for (int z = 0; z < Alignment.values().length; z++)
        {
        	final Alignment align = Alignment.values()[z];
        	alignBtns[z] = this.add(new ActButton(width - 400 + (z % 3) * 131, height - 198 + 23 * (z / 3), 129, 20, new ActButton.Callback() {
    			public void use(ActButton elem, Mode action) {
                	if(GuiSkin.this.gm.thePlayer != null) {
                		GuiSkin.this.gm.thePlayer.sendQueue.addToSendQueue(new CPacketAction(CPacketAction.Action.SET_ALIGN, align.ordinal()));
                        for(ActButton btn : alignBtns) {
                        	btn.enabled = btn != elem;
                        }
                	}
    			}
    		}, align.color + align.display));
        	alignBtns[z].enabled = this.gm.thePlayer == null || this.gm.thePlayer.getAlignment() != align;
        }
        this.add(new Slider(width - 400, height - 28 - 84, 390, 20, 1, 120, 320, 180, this.gm.thePlayer == null ? 180 : (int)(this.gm.thePlayer.getSpecies().size * this.gm.thePlayer.getHeight() * 100.0f + 0.5f), new Slider.Callback() {
			public void use(Slider elem, int value) {
				if(GuiSkin.this.gm.thePlayer != null)
					GuiSkin.this.gm.thePlayer.sendQueue.addToSendQueue(new CPacketAction(CPacketAction.Action.SET_HEIGHT, value));
			}
		}, "Spieler-Größe", "cm"));
		this.add(new Label(width - 400, height + 2 - 28 - 24 - 30, 390, 20, "Anzeigename", true));
        this.add(new ActButton(width - 274 + 154, height - 24, 264 - 154, 20, GuiMenu.INSTANCE, "Fertig"));
    	Textbox nameField = this.add(new Textbox(width - 400, height + 2 - 28 - 34, 390, 20, NetHandlerPlayServer.MAX_NICK_LENGTH, true, new Textbox.Callback() {
			public void use(Textbox elem, Action value) {
				if(value == Action.SEND || value == Action.UNFOCUS) {
					String name = elem.getText();
			    	if(name.isEmpty())
			    		elem.setText(GuiSkin.this.gm.thePlayer == null ? "..." : GuiSkin.this.gm.thePlayer.getCustomNameTag());
			    	else if(GuiSkin.this.gm.thePlayer != null)
			    		GuiSkin.this.gm.thePlayer.sendQueue.addToSendQueue(new CPacketMessage(CPacketMessage.Type.DISPLAY, name));
				}
			}
		}, NetHandlerPlayServer.VALID_NICK, this.gm.thePlayer == null ? "" : this.gm.thePlayer.getCustomNameTag()));
        this.convertButton1.enabled = false;
        this.convertButton2.enabled = false;
        this.templateButton.enabled = false;
    }
    
    public void selectSkin(BufferedImage img, ModelType model)
    {
    	this.gm.getNetHandler().addToSendQueue(new CPacketSkin(img, model));
    }
    
    public void onGuiClosed()
    {
        this.unload();
    }
    
    public void drawOverlays()
    {
        drawEntity(400 + (this.gm.fb_x - 400 - 400) / 2, this.gm.fb_y / 2 + 160, 160.0f 
        		/ this.gm.thePlayer.getHeight(), this.yaw, this.pitch, this.gm.thePlayer);
    }
    
	public String getTitle() {
		return "Charakter anpassen";
	}
	
    public static BufferedImage loadSkin(File file)
    {
    	BufferedImage img = null;
        try {
        	img = ImageIO.read(file);
        }
        catch(IOException e) {
        	return null;
        }
        if(img == null) {
        	return null;
        }
//        if(img.getWidth() != 64 || img.getHeight() != 64) { // || img.getType() != BufferedImage.TYPE_INT_ARGB) {
//        	return null;
//        }
        return img;
    }

    public void load(String currentSkin, ModelType model)
    {
        this.unload();
        File[] files = new File("skins").listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.isFile() && pathname.getName().endsWith(".png");
			}
		});
        int pos = 0;
//        for(ModelType model : ModelType.values()) {
    	this.elements.add(new SkinEntry("default", null, null, null, model));
		if("default".equals(currentSkin))
			this.setSelected(pos);
		pos++;
//        }
        if(files != null) {
	        Arrays.sort(files);
	        for(File file : files)
	        {
	        	BufferedImage img = loadSkin(file);
	        	if(img != null) {
//	        		for(ModelType model : ModelType.values()) {
        			if(img.getWidth() == model.texWidth && img.getHeight() == model.texHeight) {
		        		this.elements.add(new SkinEntry(file.getName(), file, null, img, model));
		        		if(file.getName().equals(currentSkin))
		        			this.setSelected(pos);
		        		pos++;
        			}
//	        		}
	        	}
	        }
        }
        for(SpeciesInfo species : SpeciesRegistry.SPECIMEN) {
	        for(CharacterInfo charinfo : species.chars) {
	        	if(charinfo.species.renderer == model) {
	//        for(Entry<String, ModelType> entry : SpeciesRegistry.SKINS.entrySet()) {
		        	this.elements.add(new SkinEntry(charinfo.skin, null, charinfo, null, charinfo.species.renderer));
		    		if(charinfo.skin.equals(currentSkin))
		    			this.setSelected(pos);
		    		pos++;
	        	}
	        }
        }
    }

    public void unload()
    {
        for (SkinEntry entry : this.elements)
        {
            entry.deleteTexture();
        }
        
        this.elements.clear();
    }

    public int getListWidth()
    {
        return 400 - 20;
    }
    
    public int getSlotHeight()
    {
        return 64 + 4;
    }

    
//    public static void drawEntityOnScreen(int posX, int posY, float scale, float mouseX, float mouseY, EntityLiving ent)
//    {
//        GlState.enableColorMaterial();
//        SKC.glPushMatrix();
//        SKC.glTranslatef((float)posX, (float)posY, 50.0F);
//        SKC.glScalef(-scale, scale, scale);
//        SKC.glRotatef(180.0F, 0.0F, 0.0F, 1.0F);
//        float f = ent.yawOffset;
//        float f1 = ent.rotYaw;
//        float f2 = ent.rotPitch;
//        float f3 = ent.prevHeadYaw;
//        float f4 = ent.headYaw;
//        SKC.glRotatef(135.0F, 0.0F, 1.0F, 0.0F);
//        ItemRenderer.enableStandardItemLighting();
//        SKC.glRotatef(-135.0F, 0.0F, 1.0F, 0.0F);
//        SKC.glRotatef(-((float)Math.atan((double)(mouseY / 40.0F))) * 20.0F, 1.0F, 0.0F, 0.0F);
//        ent.yawOffset = (float)Math.atan((double)(mouseX / 40.0F)) * 20.0F;
//        ent.rotYaw = (float)Math.atan((double)(mouseX / 40.0F)) * 40.0F;
//        ent.rotPitch = -((float)Math.atan((double)(mouseY / 40.0F))) * 20.0F;
//        ent.headYaw = ent.rotYaw;
//        ent.prevHeadYaw = ent.rotYaw;
//        SKC.glTranslatef(0.0F, 0.0F, 0.0F);
//        RenderManager rendermanager = Game.getGame().getRenderManager();
//        rendermanager.setPlayerViewY(180.0F);
////        rendermanager.setRenderShadow(false);
//        rendermanager.renderEntity(ent, 0.0D, 0.0D, 0.0D, 1.0F);
////        rendermanager.setRenderShadow(true);
//        ent.yawOffset = f;
//        ent.rotYaw = f1;
//        ent.rotPitch = f2;
//        ent.prevHeadYaw = f3;
//        ent.headYaw = f4;
//        SKC.glPopMatrix();
//        ItemRenderer.disableStandardItemLighting();
//        GlState.disableRescaleNormal();
//        GlState.setActiveTexture(SKC.GL_TEXTURE1);
//        GlState.disableTexture2D();
//        GlState.setActiveTexture(SKC.GL_TEXTURE0);
//    }
    
    public static void drawEntity(int posX, int posY, float scale, float yaw, float pitch, EntityLiving ent)
    {
		GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
    	GlState.enableDepth();
        GlState.enableColorMaterial();
        GL11.glPushMatrix();
        GL11.glTranslatef((float)posX, (float)posY, 200.0F);
        GL11.glScalef(-scale, scale, scale);
        GL11.glRotatef(180.0F, 0.0F, 0.0F, 1.0F);
        float f = ent.yawOffset;
        float f1 = ent.rotYaw;
        float f2 = ent.rotPitch;
        float f3 = ent.prevHeadYaw;
        float f4 = ent.headYaw;
        GL11.glRotatef(135.0F, 0.0F, 1.0F, 0.0F);
        ItemRenderer.enableStandardItemLighting();
        GL11.glRotatef(-135.0F, 0.0F, 1.0F, 0.0F);
        GL11.glTranslatef(0.0F, ent.height / 2, 0.0F);
        GL11.glRotatef(-pitch, 1.0F, 0.0F, 0.0F);
        ent.yawOffset = yaw;
        ent.rotYaw = yaw;
        ent.rotPitch = 0.0f;
        ent.headYaw = ent.rotYaw;
        ent.prevHeadYaw = ent.rotYaw;
        GL11.glTranslatef(0.0F, -(ent.height / 2), 0.0F);
        RenderManager rendermanager = Game.getGame().getRenderManager();
        rendermanager.setPlayerViewY(180.0F);
        rendermanager.renderEntity(ent, 0.0D, 0.0D, 0.0D, 1.0F);
//        GL11.glTranslatef(0.0F, 0.0F, 0.0F);
        ent.yawOffset = f;
        ent.rotYaw = f1;
        ent.rotPitch = f2;
        ent.prevHeadYaw = f3;
        ent.headYaw = f4;
        GL11.glPopMatrix();
        ItemRenderer.disableStandardItemLighting();
        GlState.disableRescaleNormal();
        GlState.setActiveTexture(GL13.GL_TEXTURE1);
        GlState.disableTexture2D();
        GlState.setActiveTexture(GL13.GL_TEXTURE0);
        GlState.disableDepth();
    }
}

